/********************************************************************************
** Form generated from reading UI file 'restoredialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RESTOREDIALOG_H
#define UI_RESTOREDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_RestoreDialog
{
public:
    QVBoxLayout *mTopLevelVLayout;
    QStackedWidget *mStackedWidget;
    QWidget *mRestoreLocationPage;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer;
    QVBoxLayout *verticalLayout_4;
    QSpacerItem *verticalSpacer_2;
    QPushButton *mRestoreOriginalButton;
    QSpacerItem *verticalSpacer_3;
    QPushButton *mRestoreCustomButton;
    QSpacerItem *verticalSpacer;
    QSpacerItem *horizontalSpacer_2;
    QWidget *mDestinationPage;
    QVBoxLayout *mDestinationVLayout;
    QHBoxLayout *mDestinationHLayout;
    QSpacerItem *horizontalSpacer_5;
    QPushButton *mDestBackButton;
    QPushButton *mDestNextButton;
    QWidget *mConfirmOverwritePage;
    QVBoxLayout *verticalLayout_6;
    QLabel *mConflictTitleLabel;
    QRadioButton *mNewNameRadioButton;
    QHBoxLayout *horizontalLayout_6;
    QSpacerItem *horizontalSpacer_9;
    QLineEdit *mNewFolderNameEdit;
    QRadioButton *mOverwriteRadioButton;
    QHBoxLayout *horizontalLayout_5;
    QSpacerItem *horizontalSpacer_8;
    QVBoxLayout *mOverwriteListboxLayout;
    QLabel *mConfirmOverwriteLabel;
    QListWidget *mFileConflictList;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *mOverwriteBackButton;
    QPushButton *mConfirmButton;
    QWidget *mRestorationProgressPage;
    QVBoxLayout *mRestorationVLayout;
    QStackedWidget *mRestorationStackWidget;
    QWidget *page;
    QVBoxLayout *mRestoreProgressLayout;
    QSpacerItem *verticalSpacer_4;
    QLabel *label_2;
    QSpacerItem *verticalSpacer_5;
    QWidget *page_2;
    QVBoxLayout *verticalLayout;
    QLabel *mErrorLabel;
    QPlainTextEdit *mRestorationOutput;
    QWidget *page_3;
    QVBoxLayout *verticalLayout_5;
    QSpacerItem *verticalSpacer_6;
    QLabel *label;
    QSpacerItem *verticalSpacer_8;
    QHBoxLayout *horizontalLayout_4;
    QSpacerItem *horizontalSpacer_7;
    QPushButton *mOpenDestinationButton;
    QSpacerItem *horizontalSpacer_6;
    QSpacerItem *verticalSpacer_7;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_4;
    QPushButton *mCloseButton;
    QWidget *mSourceScanProgressPage;
    QVBoxLayout *mSourceScanLayout;
    QSpacerItem *verticalSpacer_9;
    QLabel *label_3;
    QSpacerItem *verticalSpacer_10;

    void setupUi(QDialog *RestoreDialog)
    {
        if (RestoreDialog->objectName().isEmpty())
            RestoreDialog->setObjectName("RestoreDialog");
        RestoreDialog->resize(600, 400);
        mTopLevelVLayout = new QVBoxLayout(RestoreDialog);
        mTopLevelVLayout->setObjectName("mTopLevelVLayout");
        mStackedWidget = new QStackedWidget(RestoreDialog);
        mStackedWidget->setObjectName("mStackedWidget");
        mRestoreLocationPage = new QWidget();
        mRestoreLocationPage->setObjectName("mRestoreLocationPage");
        horizontalLayout_3 = new QHBoxLayout(mRestoreLocationPage);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_2);

        mRestoreOriginalButton = new QPushButton(mRestoreLocationPage);
        mRestoreOriginalButton->setObjectName("mRestoreOriginalButton");

        verticalLayout_4->addWidget(mRestoreOriginalButton);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_4->addItem(verticalSpacer_3);

        mRestoreCustomButton = new QPushButton(mRestoreLocationPage);
        mRestoreCustomButton->setObjectName("mRestoreCustomButton");

        verticalLayout_4->addWidget(mRestoreCustomButton);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);


        horizontalLayout_3->addLayout(verticalLayout_4);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);

        mStackedWidget->addWidget(mRestoreLocationPage);
        mDestinationPage = new QWidget();
        mDestinationPage->setObjectName("mDestinationPage");
        mDestinationVLayout = new QVBoxLayout(mDestinationPage);
        mDestinationVLayout->setObjectName("mDestinationVLayout");
        mDestinationHLayout = new QHBoxLayout();
        mDestinationHLayout->setObjectName("mDestinationHLayout");
        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        mDestinationHLayout->addItem(horizontalSpacer_5);

        mDestBackButton = new QPushButton(mDestinationPage);
        mDestBackButton->setObjectName("mDestBackButton");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("arrow-left");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        mDestBackButton->setIcon(icon);

        mDestinationHLayout->addWidget(mDestBackButton);

        mDestNextButton = new QPushButton(mDestinationPage);
        mDestNextButton->setObjectName("mDestNextButton");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("arrow-right");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        mDestNextButton->setIcon(icon1);

        mDestinationHLayout->addWidget(mDestNextButton);


        mDestinationVLayout->addLayout(mDestinationHLayout);

        mStackedWidget->addWidget(mDestinationPage);
        mConfirmOverwritePage = new QWidget();
        mConfirmOverwritePage->setObjectName("mConfirmOverwritePage");
        verticalLayout_6 = new QVBoxLayout(mConfirmOverwritePage);
        verticalLayout_6->setObjectName("verticalLayout_6");
        mConflictTitleLabel = new QLabel(mConfirmOverwritePage);
        mConflictTitleLabel->setObjectName("mConflictTitleLabel");
        QFont font;
        font.setBold(true);
        mConflictTitleLabel->setFont(font);

        verticalLayout_6->addWidget(mConflictTitleLabel);

        mNewNameRadioButton = new QRadioButton(mConfirmOverwritePage);
        mNewNameRadioButton->setObjectName("mNewNameRadioButton");
        mNewNameRadioButton->setChecked(true);

        verticalLayout_6->addWidget(mNewNameRadioButton);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        horizontalSpacer_9 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_9);

        mNewFolderNameEdit = new QLineEdit(mConfirmOverwritePage);
        mNewFolderNameEdit->setObjectName("mNewFolderNameEdit");

        horizontalLayout_6->addWidget(mNewFolderNameEdit);


        verticalLayout_6->addLayout(horizontalLayout_6);

        mOverwriteRadioButton = new QRadioButton(mConfirmOverwritePage);
        mOverwriteRadioButton->setObjectName("mOverwriteRadioButton");

        verticalLayout_6->addWidget(mOverwriteRadioButton);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalSpacer_8 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_8);

        mOverwriteListboxLayout = new QVBoxLayout();
        mOverwriteListboxLayout->setObjectName("mOverwriteListboxLayout");
        mConfirmOverwriteLabel = new QLabel(mConfirmOverwritePage);
        mConfirmOverwriteLabel->setObjectName("mConfirmOverwriteLabel");
        mConfirmOverwriteLabel->setEnabled(false);
        mConfirmOverwriteLabel->setWordWrap(true);

        mOverwriteListboxLayout->addWidget(mConfirmOverwriteLabel);

        mFileConflictList = new QListWidget(mConfirmOverwritePage);
        mFileConflictList->setObjectName("mFileConflictList");
        mFileConflictList->setEnabled(false);

        mOverwriteListboxLayout->addWidget(mFileConflictList);


        horizontalLayout_5->addLayout(mOverwriteListboxLayout);


        verticalLayout_6->addLayout(horizontalLayout_5);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);

        mOverwriteBackButton = new QPushButton(mConfirmOverwritePage);
        mOverwriteBackButton->setObjectName("mOverwriteBackButton");
        mOverwriteBackButton->setIcon(icon);

        horizontalLayout->addWidget(mOverwriteBackButton);

        mConfirmButton = new QPushButton(mConfirmOverwritePage);
        mConfirmButton->setObjectName("mConfirmButton");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("dialog-ok-apply");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        mConfirmButton->setIcon(icon2);

        horizontalLayout->addWidget(mConfirmButton);


        verticalLayout_6->addLayout(horizontalLayout);

        mStackedWidget->addWidget(mConfirmOverwritePage);
        mRestorationProgressPage = new QWidget();
        mRestorationProgressPage->setObjectName("mRestorationProgressPage");
        mRestorationVLayout = new QVBoxLayout(mRestorationProgressPage);
        mRestorationVLayout->setObjectName("mRestorationVLayout");
        mRestorationStackWidget = new QStackedWidget(mRestorationProgressPage);
        mRestorationStackWidget->setObjectName("mRestorationStackWidget");
        page = new QWidget();
        page->setObjectName("page");
        mRestoreProgressLayout = new QVBoxLayout(page);
        mRestoreProgressLayout->setObjectName("mRestoreProgressLayout");
        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        mRestoreProgressLayout->addItem(verticalSpacer_4);

        label_2 = new QLabel(page);
        label_2->setObjectName("label_2");
        label_2->setAlignment(Qt::AlignCenter);

        mRestoreProgressLayout->addWidget(label_2);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        mRestoreProgressLayout->addItem(verticalSpacer_5);

        mRestorationStackWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        verticalLayout = new QVBoxLayout(page_2);
        verticalLayout->setObjectName("verticalLayout");
        mErrorLabel = new QLabel(page_2);
        mErrorLabel->setObjectName("mErrorLabel");

        verticalLayout->addWidget(mErrorLabel);

        mRestorationOutput = new QPlainTextEdit(page_2);
        mRestorationOutput->setObjectName("mRestorationOutput");
        mRestorationOutput->setReadOnly(true);

        verticalLayout->addWidget(mRestorationOutput);

        mRestorationStackWidget->addWidget(page_2);
        page_3 = new QWidget();
        page_3->setObjectName("page_3");
        verticalLayout_5 = new QVBoxLayout(page_3);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalSpacer_6 = new QSpacerItem(20, 149, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_6);

        label = new QLabel(page_3);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignCenter);

        verticalLayout_5->addWidget(label);

        verticalSpacer_8 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_5->addItem(verticalSpacer_8);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalSpacer_7 = new QSpacerItem(192, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_7);

        mOpenDestinationButton = new QPushButton(page_3);
        mOpenDestinationButton->setObjectName("mOpenDestinationButton");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("folder");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        mOpenDestinationButton->setIcon(icon3);

        horizontalLayout_4->addWidget(mOpenDestinationButton);

        horizontalSpacer_6 = new QSpacerItem(192, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_6);


        verticalLayout_5->addLayout(horizontalLayout_4);

        verticalSpacer_7 = new QSpacerItem(20, 149, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_7);

        mRestorationStackWidget->addWidget(page_3);

        mRestorationVLayout->addWidget(mRestorationStackWidget);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_4);

        mCloseButton = new QPushButton(mRestorationProgressPage);
        mCloseButton->setObjectName("mCloseButton");
        QIcon icon4;
        iconThemeName = QString::fromUtf8("dialog-close");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        mCloseButton->setIcon(icon4);

        horizontalLayout_2->addWidget(mCloseButton);


        mRestorationVLayout->addLayout(horizontalLayout_2);

        mStackedWidget->addWidget(mRestorationProgressPage);
        mSourceScanProgressPage = new QWidget();
        mSourceScanProgressPage->setObjectName("mSourceScanProgressPage");
        mSourceScanLayout = new QVBoxLayout(mSourceScanProgressPage);
        mSourceScanLayout->setObjectName("mSourceScanLayout");
        verticalSpacer_9 = new QSpacerItem(20, 176, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        mSourceScanLayout->addItem(verticalSpacer_9);

        label_3 = new QLabel(mSourceScanProgressPage);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignCenter);

        mSourceScanLayout->addWidget(label_3);

        verticalSpacer_10 = new QSpacerItem(20, 176, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        mSourceScanLayout->addItem(verticalSpacer_10);

        mStackedWidget->addWidget(mSourceScanProgressPage);

        mTopLevelVLayout->addWidget(mStackedWidget);

        QWidget::setTabOrder(mRestoreOriginalButton, mRestoreCustomButton);
        QWidget::setTabOrder(mRestoreCustomButton, mDestNextButton);
        QWidget::setTabOrder(mDestNextButton, mOverwriteBackButton);
        QWidget::setTabOrder(mOverwriteBackButton, mDestBackButton);
        QWidget::setTabOrder(mDestBackButton, mFileConflictList);
        QWidget::setTabOrder(mFileConflictList, mConfirmButton);
        QWidget::setTabOrder(mConfirmButton, mRestorationOutput);
        QWidget::setTabOrder(mRestorationOutput, mCloseButton);

        retranslateUi(RestoreDialog);
        QObject::connect(mCloseButton, &QPushButton::clicked, RestoreDialog, qOverload<>(&QDialog::accept));
        QObject::connect(mOverwriteRadioButton, &QRadioButton::toggled, mFileConflictList, &QListWidget::setEnabled);
        QObject::connect(mOverwriteRadioButton, &QRadioButton::toggled, mConfirmOverwriteLabel, &QLabel::setEnabled);
        QObject::connect(mNewNameRadioButton, &QRadioButton::toggled, mNewFolderNameEdit, &QLineEdit::setEnabled);

        mStackedWidget->setCurrentIndex(0);
        mRestorationStackWidget->setCurrentIndex(0);
        mCloseButton->setDefault(true);


        QMetaObject::connectSlotsByName(RestoreDialog);
    } // setupUi

    void retranslateUi(QDialog *RestoreDialog)
    {
        RestoreDialog->setWindowTitle(tr2i18n("Restore Guide", "@title:window"));
        mRestoreOriginalButton->setText(tr2i18n("Restore to original location", "@action:button"));
        mRestoreCustomButton->setText(tr2i18n("Choose where to restore", "@action:button"));
        mDestBackButton->setText(tr2i18n("Back", "@action:button"));
        mDestNextButton->setText(tr2i18n("Next", "@action:button"));
        mConflictTitleLabel->setText(QString());
        mNewNameRadioButton->setText(tr2i18n("Restore the folder under a new name", "@option:radio "));
        mOverwriteRadioButton->setText(tr2i18n("Merge folders", "@option:radio"));
        mConfirmOverwriteLabel->setText(tr2i18n("The following files would be overwritten, please confirm that you wish to continue.", "@info Question to user, in dialog"));
        mOverwriteBackButton->setText(tr2i18n("Back", "@action:button go to previous page in dialog"));
        mConfirmButton->setText(tr2i18n("Confirm", "@action:button User answers \"yes I confirm: overwrite files\""));
        label_2->setText(tr2i18n("Restoring files", "Title above progress bar"));
        mErrorLabel->setText(tr2i18n("An error occurred while restoring:", "@label above the detailed error message"));
        label->setText(tr2i18n("Restoration completed successfully!", "@label"));
        mOpenDestinationButton->setText(tr2i18n("Open Destination", nullptr));
        mCloseButton->setText(tr2i18n("Close", "@action:button"));
        label_3->setText(tr2i18n("Checking file sizes", "Title above progress bar"));
    } // retranslateUi

};

namespace Ui {
    class RestoreDialog: public Ui_RestoreDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // RESTOREDIALOG_H

