/****************************************************************************
** Meta object code from reading C++ file 'planexecutor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kup-0.10.0/daemon/planexecutor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'planexecutor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12PlanExecutorE_t {};
} // unnamed namespace

template <> constexpr inline auto PlanExecutor::qt_create_metaobjectdata<qt_meta_tag_ZN12PlanExecutorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlanExecutor",
        "stateChanged",
        "",
        "backupStatusChanged",
        "checkStatus",
        "showBackupFiles",
        "showBackupPurger",
        "updateAccumulatedUsageTime",
        "startIntegrityCheck",
        "startRepairJob",
        "startBackupSaveJob",
        "showLog",
        "startBackup",
        "finishBackup",
        "KJob*",
        "pJob",
        "finishSizeCheck",
        "exitBackupRunningState",
        "pWasSuccessful",
        "enterAvailableState",
        "askUserOrStart",
        "pUserQuestion",
        "enterNotAvailableState",
        "askUser",
        "pQuestion",
        "discardUserQuestion",
        "notifyBackupFailed",
        "pFailedJob",
        "discardFailNotification",
        "notifyBackupSucceeded",
        "integrityCheckFinished",
        "discardIntegrityNotification",
        "repairFinished",
        "discardRepairNotification",
        "startSleepInhibit",
        "endSleepInhibit"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'backupStatusChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'checkStatus'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showBackupFiles'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showBackupPurger'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateAccumulatedUsageTime'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startIntegrityCheck'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startRepairJob'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startBackupSaveJob'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showLog'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startBackup'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'finishBackup'
        QtMocHelpers::SlotData<void(KJob *)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'finishSizeCheck'
        QtMocHelpers::SlotData<void(KJob *)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'exitBackupRunningState'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Slot 'enterAvailableState'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'askUserOrStart'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'enterNotAvailableState'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'askUser'
        QtMocHelpers::SlotData<void(const QString &)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Slot 'discardUserQuestion'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyBackupFailed'
        QtMocHelpers::SlotData<void(KJob *)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 27 },
        }}),
        // Slot 'discardFailNotification'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'notifyBackupSucceeded'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'integrityCheckFinished'
        QtMocHelpers::SlotData<void(KJob *)>(30, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'discardIntegrityNotification'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'repairFinished'
        QtMocHelpers::SlotData<void(KJob *)>(32, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'discardRepairNotification'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'startSleepInhibit'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'endSleepInhibit'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlanExecutor, qt_meta_tag_ZN12PlanExecutorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlanExecutor::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PlanExecutorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PlanExecutorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12PlanExecutorE_t>.metaTypes,
    nullptr
} };

void PlanExecutor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlanExecutor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stateChanged(); break;
        case 1: _t->backupStatusChanged(); break;
        case 2: _t->checkStatus(); break;
        case 3: _t->showBackupFiles(); break;
        case 4: _t->showBackupPurger(); break;
        case 5: _t->updateAccumulatedUsageTime(); break;
        case 6: _t->startIntegrityCheck(); break;
        case 7: _t->startRepairJob(); break;
        case 8: _t->startBackupSaveJob(); break;
        case 9: _t->showLog(); break;
        case 10: _t->startBackup(); break;
        case 11: _t->finishBackup((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 12: _t->finishSizeCheck((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 13: _t->exitBackupRunningState((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->enterAvailableState(); break;
        case 15: _t->askUserOrStart((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->enterNotAvailableState(); break;
        case 17: _t->askUser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->discardUserQuestion(); break;
        case 19: _t->notifyBackupFailed((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 20: _t->discardFailNotification(); break;
        case 21: _t->notifyBackupSucceeded(); break;
        case 22: _t->integrityCheckFinished((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 23: _t->discardIntegrityNotification(); break;
        case 24: _t->repairFinished((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 25: _t->discardRepairNotification(); break;
        case 26: _t->startSleepInhibit(); break;
        case 27: _t->endSleepInhibit(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlanExecutor::*)()>(_a, &PlanExecutor::stateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlanExecutor::*)()>(_a, &PlanExecutor::backupStatusChanged, 1))
            return;
    }
}

const QMetaObject *PlanExecutor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlanExecutor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PlanExecutorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PlanExecutor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void PlanExecutor::stateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PlanExecutor::backupStatusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
